@@ -39,6 +39,8 @@ impl ScalarOperand for i32 { }
 impl ScalarOperand for u32 { }
 impl ScalarOperand for i64 { }
 impl ScalarOperand for u64 { }
+impl ScalarOperand for i128 { }
+impl ScalarOperand for u128 { }
 impl ScalarOperand for isize { }
 impl ScalarOperand for usize { }
 impl ScalarOperand for f32 { }
@@ -105,7 +107,7 @@ impl<'a, A, S, S2, D, E> $trt<&'a ArrayBase<S2, E>> for ArrayBase<S, D>
 /// If their shapes disagree, `rhs` is broadcast to the shape of `self`.
 ///
 /// **Panics** if broadcasting isn’t possible.
-impl<'a, 'b, A, S, S2, D, E> $trt<&'a ArrayBase<S2, E>> for &'b ArrayBase<S, D>
+impl<'a, A, S, S2, D, E> $trt<&'a ArrayBase<S2, E>> for &'a ArrayBase<S, D>
     where A: Clone + $trt<A, Output=A>,
           S: Data<Elem=A>,
           S2: Data<Elem=A>,
@@ -250,6 +252,8 @@ mod arithmetic_ops {
     all_scalar_ops!(u32);
     all_scalar_ops!(i64);
     all_scalar_ops!(u64);
+    all_scalar_ops!(i128);
+    all_scalar_ops!(u128);
 
     impl_scalar_lhs_op!(bool, Commute, &, BitAnd, bitand, "bit and");
     impl_scalar_lhs_op!(bool, Commute, |, BitOr, bitor, "bit or");
