@@ -82,6 +82,14 @@ fn test_slice()
     assert!(vi.iter().zip(A.iter()).all(|(a, b)| a == b));
 }
 
+#[test]
+fn test_slice_inclusive_range() {
+    let arr = array![[1, 2, 3], [4, 5, 6]];
+    assert_eq!(arr.slice(s![1..=1, 1..=2]), array![[5, 6]]);
+    assert_eq!(arr.slice(s![1..=-1, -2..=2;-1]), array![[6, 5]]);
+    assert_eq!(arr.slice(s![0..=-1, 0..=2;2]), array![[1, 3], [4, 6]]);
+}
+
 /// Test that the compiler can infer a type for a sliced array from the
 /// arguments to `s![]`.
 ///
@@ -274,28 +282,37 @@ fn test_slice_inplace_with_subview_inplace() {
         *elt = i;
     }
 
-    let mut vi = arr.view();
-    vi.slice_inplace(s![1.., 2, ..;2]);
-    assert_eq!(vi.shape(), &[2, 1, 2]);
-    assert!(
-        vi.iter()
-            .zip(arr.slice(s![1.., 2..3, ..;2]).iter())
-            .all(|(a, b)| a == b)
-    );
+    {
+        let mut vi = arr.view();
+        vi.slice_inplace(s![1.., 2, ..;2]);
+        assert_eq!(vi.shape(), &[2, 1, 2]);
+        assert!(
+            vi.iter()
+                .zip(arr.slice(s![1.., 2..3, ..;2]).iter())
+                .all(|(a, b)| a == b)
+        );
 
-    let mut vi = arr.view();
-    vi.slice_inplace(s![1, 2, ..;2]);
-    assert_eq!(vi.shape(), &[1, 1, 2]);
-    assert!(
-        vi.iter()
-            .zip(arr.slice(s![1..2, 2..3, ..;2]).iter())
-            .all(|(a, b)| a == b)
-    );
+        let mut vi = arr.view();
+        vi.slice_inplace(s![1, 2, ..;2]);
+        assert_eq!(vi.shape(), &[1, 1, 2]);
+        assert!(
+            vi.iter()
+                .zip(arr.slice(s![1..2, 2..3, ..;2]).iter())
+                .all(|(a, b)| a == b)
+        );
 
-    let mut vi = arr.view();
+        let mut vi = arr.view();
+        vi.slice_inplace(s![1, 2, 3]);
+        assert_eq!(vi.shape(), &[1, 1, 1]);
+        assert_eq!(vi, Array3::from_elem((1, 1, 1), arr[(1, 2, 3)]));
+    }
+
+    // Do it to the RcArray itself
+    let elem = arr[(1, 2, 3)];
+    let mut vi = arr;
     vi.slice_inplace(s![1, 2, 3]);
     assert_eq!(vi.shape(), &[1, 1, 1]);
-    assert_eq!(vi, Array3::from_elem((1, 1, 1), arr[(1, 2, 3)]));
+    assert_eq!(vi, Array3::from_elem((1, 1, 1), elem));
 }
 
 #[should_panic]
@@ -680,6 +697,68 @@ fn sum_mean()
     assert_eq!(a.scalar_sum(), 10.);
 }
 
+#[test]
+fn var_axis() {
+    let a = array![
+        [
+            [-9.76, -0.38, 1.59, 6.23],
+            [-8.57, -9.27, 5.76, 6.01],
+            [-9.54, 5.09, 3.21, 6.56],
+        ],
+        [
+            [ 8.23, -9.63, 3.76, -3.48],
+            [-5.46, 5.86, -2.81, 1.35],
+            [-1.08, 4.66, 8.34, -0.73],
+        ],
+    ];
+    assert!(a.var_axis(Axis(0), 1.5).all_close(
+        &aview2(&[
+            [3.236401e+02, 8.556250e+01, 4.708900e+00, 9.428410e+01],
+            [9.672100e+00, 2.289169e+02, 7.344490e+01, 2.171560e+01],
+            [7.157160e+01, 1.849000e-01, 2.631690e+01, 5.314410e+01]
+        ]),
+        1e-4,
+    ));
+    assert!(a.var_axis(Axis(1), 1.7).all_close(
+        &aview2(&[
+            [0.61676923, 80.81092308, 6.79892308, 0.11789744],
+            [75.19912821, 114.25235897, 48.32405128, 9.03020513],
+        ]),
+        1e-8,
+    ));
+    assert!(a.var_axis(Axis(2), 2.3).all_close(
+        &aview2(&[
+            [ 79.64552941, 129.09663235, 95.98929412],
+            [109.64952941, 43.28758824, 36.27439706],
+        ]),
+        1e-8,
+    ));
+
+    let b = array![[1.1, 2.3, 4.7]];
+    assert!(b.var_axis(Axis(0), 0.).all_close(&aview1(&[0., 0., 0.]), 1e-12));
+    assert!(b.var_axis(Axis(1), 0.).all_close(&aview1(&[2.24]), 1e-12));
+
+    let c = array![[], []];
+    assert_eq!(c.var_axis(Axis(0), 0.), aview1(&[]));
+
+    let d = array![1.1, 2.7, 3.5, 4.9];
+    assert!(d.var_axis(Axis(0), 0.).all_close(&aview0(&1.8875), 1e-12));
+}
+
+#[test]
+#[should_panic]
+fn var_axis_bad_dof() {
+    let a = array![1., 2., 3.];
+    a.var_axis(Axis(0), 4.);
+}
+
+#[test]
+#[should_panic]
+fn var_axis_empty_axis() {
+    let a = array![[], []];
+    a.var_axis(Axis(1), 0.);
+}
+
 #[test]
 fn iter_size_hint()
 {
@@ -807,6 +886,44 @@ fn owned_array_with_stride() {
     assert_eq!(a.strides(), &[1, 4, 2]);
 }
 
+#[test]
+fn owned_array_discontiguous() {
+    use ::std::iter::repeat;
+    let v: Vec<_> = (0..12).flat_map(|x| repeat(x).take(2)).collect();
+    let dim = (3, 2, 2);
+    let strides = (8, 4, 2);
+
+    let a = Array::from_shape_vec(dim.strides(strides), v).unwrap();
+    assert_eq!(a.strides(), &[8, 4, 2]);
+    println!("{:?}", a.iter().cloned().collect::<Vec<_>>());
+    itertools::assert_equal(a.iter().cloned(), 0..12);
+}
+
+#[test]
+fn owned_array_discontiguous_drop() {
+    use ::std::rc::Rc;
+    use ::std::cell::RefCell;
+    use ::std::collections::BTreeSet;
+
+    struct InsertOnDrop<T: Ord>(Rc<RefCell<BTreeSet<T>>>, Option<T>);
+    impl<T: Ord> Drop for InsertOnDrop<T> {
+        fn drop(&mut self) {
+            let InsertOnDrop(ref set, ref mut value) = *self;
+            set.borrow_mut().insert(value.take().expect("double drop!"));
+        }
+    }
+
+    let set = Rc::new(RefCell::new(BTreeSet::new()));
+    {
+        let v: Vec<_> = (0..12).map(|x| InsertOnDrop(set.clone(), Some(x))).collect();
+        let mut a = Array::from_shape_vec((2, 6), v).unwrap();
+        // discontiguous and non-zero offset
+        a.slice_inplace(s![.., 1..]);
+    }
+    // each item was dropped exactly once
+    itertools::assert_equal(set.borrow().iter().cloned(), 0..12);
+}
+
 macro_rules! assert_matches {
     ($value:expr, $pat:pat) => {
         match $value {
@@ -1372,6 +1489,18 @@ fn to_owned_neg_stride() {
     assert_eq!(c, co);
 }
 
+#[test]
+fn discontiguous_owned_to_owned() {
+    let mut c = arr2(&[[1, 2, 3],
+                       [4, 5, 6]]);
+    c.slice_inplace(s![.., ..;2]);
+
+    let co = c.to_owned();
+    assert_eq!(c.strides(), &[3, 2]);
+    assert_eq!(co.strides(), &[2, 1]);
+    assert_eq!(c, co);
+}
+
 #[test]
 fn map_memory_order() {
     let a = arr3(&[[[1, 2, 3],
